// -----------------------------------------------------------------------
// Controller
//
Ext.define('PO.tsweekly.HourWeeklyController', {
    extend: 'Ext.app.Controller',
    id: 'hourWeeklyController',
    
    // Parameters
    debug: true,
    hourButtonPanel: null,
    hourController: null,
    hourGrid: null,
    taskTreeStore: null,
    rowEditing: null,
    hourWeeklyStore: null,

    // Setup the various listeners to receive events here
    init: function() {
        var me = this;
        if (me.debug) { console.log('PO.tsweekly.HourWeeklyController.init: Starting'); }

        this.control({
            '#buttonAddEntry': { click: this.onButtonAddEntry },
            '#buttonDeleteEntry': { click: this.onButtonDeleteEntry },
            scope: me
        });

        // Listen to the Grid Editor that allows to specify start- and end time
        me.hourGrid.on('edit', this.onGridEdit, me);
        me.hourGrid.on('beforeedit', this.onGridBeforeEdit, me);

        // Trying to crate a "change" event from the Project combo,
        // but this didn't work. Now just hard-wiring...
	// ToDo

        // Catch a global key strokes. This is used to abort entry with Esc.
        // For some reaons this doesn't work on the level of the HourButtonPanel, so we go for the global "window"
        Ext.EventManager.on(window, 'keydown', this.onWindowKeyDown, me);

        if (me.debug) { console.log('PO.tsweekly.HourWeeklyController.init: Finished'); }
        return this;
    },

    /**
     * The value of the Project field has changed in the RowEditing.
     */
    onProjectChanged: function(projectId, combo, hourGrid) {
	var me = this;
	if (me.debug) console.log('HourWeeklyConstroller.onProjectChanged: Started');

        // Load taskTreeStore with new parameter
        me.taskTreeStore.getProxy().extraParams = { project_id: projectId };
        me.taskTreeStore.reload({
            callback: function(records, operation, success) {
                var me = this;
                if (hourGrid.debug) console.log('HourWeeklyConstroller.onProjectChanged: TaskTreeStore: loaded');
                if (!success) {
                    PO.Utilities.reportStoreError("TaskTreeStore", operation);
                    return;
                }
                var mainProjectNode = records[0];
                mainProjectNode.set('sort_order','0');
                me.setRootNode(mainProjectNode);
            }
        });

	if (me.debug) console.log('HourWeeklyConstroller.onProjectChanged: Finished');
    },

    /*
     * The user has double-clicked on the row editor in order to
     * manually fill in the values. This procedure automatically
     * fills in the end_time.
     */
    onGridBeforeEdit: function(editor, context, eOpts) {
	var me = this;
	if (me.debug) console.log('HourWeeklyController.onGridBeforeEdit: Starting');
        // console.log(context.record);

        // Return true to indicate to the editor that it's OK to edit
	if (me.debug) console.log('HourWeeklyController.onGridBeforeEdit: Finished');
        return true;
    },

    /**
     * Gets called after finishing an edit using the "Update" button
     */
    onGridEdit: function(editor, context) {
	var me = this;
        if (me.debug) console.log('HourWeeklyController.onGridEdit: Starting');
        var rec = context.record;
        
        var interval_date = rec.get('interval_date');
        var interval_start = rec.get('interval_start');
        var interval_start_time = rec.get('interval_start_time');
        var interval_end = rec.get('interval_end');
        var interval_end_time = rec.get('interval_end_time');
        if ("" == interval_start_time) { interval_start_time = null; }
        if ("" == interval_end_time) { interval_end_time = null; }

        // start == end => Delete the entry
        if (interval_start_time != null && interval_end_time != null) {
            if (interval_start_time == interval_end_time) {
                rec.destroy();
                return;
            }
        }

        if (interval_date != null) {
            // The interval_date has been overwritten by the editor with a Date
            var value = new Date(interval_date);
            rec.set('interval_date', Ext.Date.format(value, 'Y-m-d'));
        }

        if (interval_date != null && interval_start_time != null) {
            var value = new Date(interval_date);
            value.setHours(interval_start_time.substring(0,2));
            value.setMinutes(interval_start_time.substring(3,5));
            rec.set('interval_start', Ext.Date.format(value, 'Y-m-d H:i:s'));
        }

        if (interval_date != null && interval_end_time != null) {
            var value = new Date(interval_date);
            value.setHours(interval_end_time.substring(0,2));
            value.setMinutes(interval_end_time.substring(3,5));
            rec.set('interval_end', Ext.Date.format(value, 'Y-m-d H:i:s'));
        }

        rec.save();
        rec.commit();

        if (me.debug) console.log('HourWeeklyController.onGridEdit: Finished');
    },

    // Esc (Escape) button pressed somewhere in the application window
    onWindowKeyDown: function(e) {
        var keyCode = e.getKey();
        var keyCtrl = e.ctrlKey;
        console.log('HourWeeklyController.onWindowKeyDown: code='+keyCode+', ctrl='+keyCtrl);
        
        // cancel hour logging with Esc key
        if (27 == keyCode) {
            // Cancel Edit
        }
        if (46 == keyCode) { this.onButtonDeleteEntry(); }
    },

    // Click into the empty space below the grid entries in order to start creating a new entry
    onGridContainerClick: function() {
	var me = this;
        if (me.debug) console.log('HourWeeklyController.GridContainerClick: Starting');

	alert('xxx');
	
        if (me.debug) console.log('HourWeeklyController.GridContainerClick: Finished');
    },

    /*
     * Start logging the time, for entirely manual entries.
     */
    onButtonAddEntry: function() {
	var me = this;
        if (me.debug) console.log('HourWeeklyController.!!! ButtonManualLogging');

        // var buttonAddEntry = Ext.getCmp('buttonAddEntry');
        // var buttonDeleteEntry = Ext.getCmp('buttonDeleteEntry');
        // buttonAddEntry.disable();
        // buttonDeleteEntry.enable();
        me.rowEditing.cancelEdit();

        var day = PO.Utilities.dateToPg(new Date());
        var hourWeekly = new Ext.create('PO.tsweekly.HourWeekly', {
	    project_id: null,
            user_id: @current_user_id@,
            day: day
        });

        me.hourWeeklyStore.add(hourWeekly);
	var proxy = me.hourWeeklyStore.getProxy();
	hourWeekly.setProxy(proxy);
	
        me.rowEditing.startEdit(hourWeekly, 0);
    },

    onButtonDeleteEntry: function() {
        var me = this;
        console.log('HourWeeklyController.ButtonDeleteLogging');

        var buttonAddEntry = Ext.getCmp('buttonAddEntry');
        // var buttonDeleteEntry = Ext.getCmp('buttonDeleteEntry');
        buttonAddEntry.enable();
        // buttonDeleteEntry.enable();
        me.rowEditing.cancelEdit();

        var records = hourGrid.getSelectionModel().getSelection();
        // Not logging already - enable the "start" button
        if (1 == records.length) {                  // Exactly one record enabled
            var record = records[0];
            me.hourWeeklyStore.remove(record);
            record.destroy();
        }
    },

    /**
     * Control the enabled/disabled status of the Start/Stop logging buttons.
     * Skip logging hours when changing the selection.
     */
    onTreePanelSelectionChange: function(view, records) {
        if (this.debug) { console.log('HourWeeklyController.onTreePanelSelectionChange'); }

        var buttonAddEntry = Ext.getCmp('buttonAddEntry');
        // Not logging already - enable the "start" button
        if (1 == records.length) {					// Exactly one record enabled

        } else {							// Zero or two or more records enabled
            buttonAddEntry.setDisabled(true);
        }                
    },

    /**
     * Clicking around in the grid part of the screen,
     * Enable or disable the "Delete" button
     */
    onGridSelectionChange: function(view, records) {
        if (this.debug) { console.log('HourWeeklyController.onGridSelectionChange'); }
        var buttonDeleteEntry = Ext.getCmp('buttonDeleteEntry');
        buttonDeleteEntry.setDisabled(1 != records.length);
    },


    /**
     * Handle various key actions
     */
    onCellKeyDown: function(table, htmlTd, cellIndex, record, htmlTr, rowIndex, e, eOpts) {
        console.log('HourWeeklyController.onCellKeyDown');
        var keyCode = e.getKey();
        var keyCtrl = e.ctrlKey;
        console.log('HourWeeklyController.onCellKeyDown: code='+keyCode+', ctrl='+keyCtrl);
    }
    
});
